@@ -20,7 +20,6 @@
 #include <fmp4/mp4_options.h>
 #include <fmp4/moov.h>
 #include <fmp4/output_bucket.h>
-#include <fmp4/mp4_handler_io.h>
 #include <fmp4/post_handler.h>
 
 #if 0
@@ -66,6 +65,7 @@ typedef struct
 
   ngx_flag_t usp_handle_f4f;
   ngx_flag_t usp_handle_api;
+  ngx_flag_t usp_handle_smpd;
   ngx_flag_t usp_disable_mmap;
 
   ngx_str_t s3_secret_key;
@@ -101,6 +101,7 @@ static char* ngx_usp_dynamic_time_shift_buffer_depth(ngx_conf_t* cf, ngx_command
 static char* ngx_usp_skip_rewrite(ngx_conf_t* cf, ngx_command_t* cmd, void* conf);
 static char* ngx_usp_handle_f4f(ngx_conf_t* cf, ngx_command_t* cmd, void* conf);
 static char* ngx_usp_handle_api(ngx_conf_t* cf, ngx_command_t* cmd, void* conf);
+static char* ngx_usp_handle_smpd(ngx_conf_t* cf, ngx_command_t* cmd, void* conf);
 static char* ngx_usp_disable_mmap(ngx_conf_t* cf, ngx_command_t* cmd, void* conf);
 static char* ngx_usp_s3_secret_key(ngx_conf_t* cf, ngx_command_t* cmd, void* conf);
 static char* ngx_usp_s3_access_key(ngx_conf_t* cf, ngx_command_t* cmd, void* conf);
@@ -211,7 +212,7 @@ static ngx_command_t ngx_streaming_commands[] =
     NULL
   },
 
-    { ngx_string("usp_handle_api"),
+  { ngx_string("usp_handle_api"),
     NGX_HTTP_LOC_CONF|NGX_CONF_NOARGS,
     ngx_usp_handle_api,
     NGX_HTTP_LOC_CONF_OFFSET,
@@ -219,6 +220,14 @@ static ngx_command_t ngx_streaming_commands[] =
     NULL
   },
 
+  { ngx_string("usp_handle_storage_mpd"),
+    NGX_HTTP_LOC_CONF|NGX_CONF_NOARGS,
+    ngx_usp_handle_smpd,
+    NGX_HTTP_LOC_CONF_OFFSET,
+    0,
+    NULL
+  },
+
   { ngx_string("usp_disable_mmap"),
     NGX_HTTP_LOC_CONF|NGX_CONF_NOARGS,
     ngx_usp_disable_mmap,
@@ -324,6 +333,7 @@ ngx_http_streaming_create_loc_conf(ngx_conf_t *cf)
   conf->usp_dynamic_time_shift_buffer_depth = NGX_CONF_UNSET;
   conf->usp_handle_f4f = NGX_CONF_UNSET;
   conf->usp_handle_api = NGX_CONF_UNSET;
+  conf->usp_handle_smpd = NGX_CONF_UNSET;
   conf->usp_disable_mmap = NGX_CONF_UNSET;
   conf->s3_secret_key.data = NULL;
   conf->s3_secret_key.len = NGX_CONF_UNSET;
@@ -348,6 +358,7 @@ ngx_http_streaming_merge_loc_conf(ngx_conf_t *cf, void *parent, void *child)
   ngx_conf_merge_value(conf->usp_skip_rewrite, prev->usp_skip_rewrite, 0);
   ngx_conf_merge_value(conf->usp_handle_f4f, prev->usp_handle_f4f, 0);
   ngx_conf_merge_value(conf->usp_handle_api, prev->usp_handle_api, 0);
+  ngx_conf_merge_value(conf->usp_handle_smpd, prev->usp_handle_smpd, 0);
   ngx_conf_merge_value(conf->usp_disable_mmap, prev->usp_disable_mmap, 0);
 
   ngx_conf_merge_str_value(conf->s3_secret_key, prev->s3_secret_key, 0);
@@ -820,7 +831,7 @@ static ngx_int_t ngx_streaming_handler(ngx_http_request_t *r)
 
   if(slcf->usp_disable_mmap)
   {
-    context->create_handler_io_ = create_handler_io_no_mmap;
+    context->usp_enable_mmap_ = 0;
   }
 
   if(slcf->s3_secret_key.data && slcf->s3_access_key.data)
@@ -855,6 +866,11 @@ static ngx_int_t ngx_streaming_handler(ngx_http_request_t *r)
     context->is_rest_api_ = 1;
   }
 
+  if(slcf->usp_handle_smpd > 0)
+  {
+    context->is_storage_mpd_ = 1;
+  }
+
   if(r->method & (NGX_HTTP_GET|NGX_HTTP_HEAD))
   {
     http_status = ngx_streaming_handler_get(r, context);
@@ -1267,6 +1283,7 @@ ngx_streaming_handler_get(ngx_http_request_t *r,
 
       if(cache_control.len)
       {
+#if nginx_version < 1023000
         ngx_table_elt_t** ccp = r->headers_out.cache_control.elts;
         ngx_table_elt_t* cc;
 
@@ -1303,6 +1320,33 @@ ngx_streaming_handler_get(ngx_http_request_t *r,
           }
           cc = ccp[0];
         }
+#else
+        ngx_table_elt_t* cc = r->headers_out.cache_control;
+
+        if(cc == NULL)
+        {
+          cc = ngx_list_push(&r->headers_out.headers);
+          if(cc == NULL)
+          {
+            return NGX_ERROR;
+          }
+
+          r->headers_out.cache_control = cc;
+          cc->next = NULL;
+
+          cc->hash = 1;
+          ngx_str_set(&cc->key, "Cache-Control");
+        }
+        else
+        {
+          for(cc = cc->next; cc; cc = cc->next)
+          {
+            cc->hash = 0;
+          }
+          cc = r->headers_out.cache_control;
+          cc->next = NULL;
+        }
+#endif
 
         cc->value = cache_control;
       }
@@ -1448,7 +1492,7 @@ ngx_streaming_handler_get(ngx_http_request_t *r,
       {
         ngx_str_t path;
         ngx_open_file_info_t of;
-        char const* filename_prev = 0;
+        char const* filename_prev = NULL;
 
         for(; bucket != head; bucket = bucket_next(bucket))
         {
@@ -1481,8 +1525,10 @@ ngx_streaming_handler_get(ngx_http_request_t *r,
             }
 
             // poor man's caching
-            if(fname != filename_prev)
+            if(filename_prev == NULL ||
+               fname != filename_prev && strcmp(fname, filename_prev) != NULL)
             {
+              //ngx_log_error(NGX_LOG_INFO, log, 0, "poor man's cache miss on %s", fname);
               filename_prev = fname;
               {
                 char* dst;
@@ -1504,6 +1550,10 @@ ngx_streaming_handler_get(ngx_http_request_t *r,
                 return http_status;
               }
             }
+            else
+            {
+              //ngx_log_error(NGX_LOG_INFO, log, 0, "poor man's cache hit on %s", fname);
+            }
 
             b->file->fd = of.fd;
             b->file->name = path;
@@ -1639,6 +1689,20 @@ static char* ngx_usp_handle_api(ngx_conf_t* cf, ngx_command_t* cmd, void* conf)
   return NGX_CONF_OK;
 }
 
+static char* ngx_usp_handle_smpd(ngx_conf_t* cf, ngx_command_t* cmd, void* conf)
+{
+  ngx_http_core_loc_conf_t* clcf;
+  ngx_http_streaming_loc_conf_t* slcf = conf;
+
+  clcf = ngx_http_conf_get_module_loc_conf(cf, ngx_http_core_module);
+  clcf->handler = ngx_streaming_handler;
+
+  slcf->enable = 1;
+  slcf->usp_handle_smpd = 1;
+
+  return NGX_CONF_OK;
+}
+
 static char* ngx_usp_disable_mmap(ngx_conf_t* cf, ngx_command_t* cmd, void* conf)
 {
   ngx_http_streaming_loc_conf_t* slcf = conf;
